module binary-fpc.

% Note: if we require the first clause in a resolution triple to be
%  the clause with the "postive" resolving literal, then some of
%  non-determinism in checking is eliminated.  See (*) in comment
%  below.  This suggests a different proof certificate format where
%  the user is requested to make such a choice.  This is likely to be
%  a recurring theme: there is a lot of non-determinism in checking
%  proofs: the checker can search to resolve it or the user can do
%  some additional work to help remove some worthless non-determinism.

type  print'   A -> o.
print' Term :- term_to_string Term String, print String.

% The clerks computation: The first set of these operate on
% the left premises of cut.
orClerk        _      (dlist L) (dlist L).
allClerk       _      (dlist L) (dlist L).
falseClerk     _      (dlist L) (dlist L).
store2Clerk C (lit C) (dlist L) (dlist L) :- print "newlit = ",  print' C, print "\n".

% These clerks operaton the the right branches of cuts.
falseClerk  _         (rlist Rs)    (rlist Rs).
store2Clerk C (idx K) (rlisti K Rs) (rlist Rs) :- print "newclause ", print' K, print " = ", print' C, print "\n".

% Stage: cut rule, with a release/store on right premise.  
% Certificates of the form (rlist (_::_)).

cutExpert NCut Empty Empty (rlist (resol I J K::Rs)) (dlist [I,J]) (rlisti K Rs)  _ :- 
  mset_empty Empty, ref (idx K) Cut, negate Cut NCut,
  print "Resolving ", print' (resol I J K),
  print " yields cut formula ", print' NCut, print ".\n".

% Stage: end of right-most premise proof.  We must have the +2 true on the 
%        left of the uparrow (this is the negation of the empty clause).
% Certificates of the form (rlist []) and done.

decideExpert (idx I) (rlist []) rdone _ :- ref (idx I) True, true+ True.
trueExpert rdone _.

% Stage: left premise proof where we check a specific binary resolution
% Certificates of the form (dlist []) and ddone.

initialExpert _ (dlist _) _.
trueExpert      (dlist _) _.
releaseExpert   (dlist L) (dlist L) _.

andExpert Empty Empty (dlist L) (dlist L) (dlist L) _ :- mset_empty Empty.
existsExpert _ (dlist L) (dlist L) _.

decideExpert (idx I) (dlist [I,J]) (dlist [J]) _.
% (*) If the certificate enters the clauses in a certain order, some
%     non-determinism can be removed with help from the user.
%     For the general (unsorted) case, the following should be uncommented.
%decideExpert (idx I) (dlist [J,I]) (dlist [J]) _.
decideExpert (idx I) (dlist [I]) (dlist [] ) _.
decideExpert (lit P) (dlist [] ) ddone _ :- ref (lit P) P, pos_lit P.
initialExpert _ ddone _.
