sig certificate.
accum_sig lkuForm, lkuSeq, msets, lku.


/* The interface between the kernel and proof certificate definitions.
 The key type certificate is declared. 
 Two collection of predicates are also declared: the clerks giving 
 the computations  to do during the asynchronous phase and the experts 
 for extracting  information from the certificate during the 
 asynchronous phase. */


kind cert              type.  



/* Computations performed during the asynchronous phase 
  The formula given as the first argument corresponds to the formula F in 
  the conclusion of the asynchronous rule:
          .
          .
          .
   ---------------
   ⊢ Γ : Δ ⇑ F, Θ  

  As we read bottom-up, the first cert corresponds to the certificate attached
  to the conclusion. The second one is computed by the clerk and the checker
  will continue with it.
  In the case of the conjunction, there are two certificates computed,
  respectively attached to the left and right premiss.

  For the two store rules, the formula will be stored by the kernel
  under the index computed by the clerk.
*/

type     andClerk      form  -> cert -> cert -> cert -> o.
type      orClerk      form          -> cert -> cert -> o.
type     allClerk      form          -> cert -> cert -> o.
type   falseClerk      form          -> cert -> cert -> o.
type  store1Clerk      form -> index -> cert -> cert -> o.
type  store2Clerk      form -> index -> cert -> cert -> o.

/* For the synchronous ors (tensor and /\+), experts have to choose 
  betwen left  and right */
kind choice            type.
type left, right       choice.


/* Experts: these extract information from certificates during the 
 synchronous phase.
 They deal with positive sequents and boundry sequents. 

 As for clerks, the first certificate is attached to the conclusion and the 
 second (or second and third) is one and extracted by the expert and the checker
 continue with it.

 For the existential expert, the first argument is the witness that the 
 expert should extract.

 For the and expert and the cut expert, the two lincon represent respectively
 Δ1 and Δ2 extracted as described by the following rule:
 ⊢ Γ : Δ1 ⇕ Θ1    ⊢ Γ : Δ1 ⇕ Θ2
 ------------------------------
        ⊢ Γ : Δ1 Δ2 ⇕ Θ 

 The last argument (seq or bseq) is the entire conclusion given to the expert
 to help it in making decisions
*/
type releaseExpert                                   cert -> cert ->  seq -> o.
type      orExpert                         choice -> cert -> cert ->  seq -> o.
type  existsExpert                              A -> cert -> cert ->  seq -> o.
type    trueExpert                                           cert ->  seq -> o.
type     andExpert       lincon -> lincon -> cert -> cert -> cert ->  seq -> o.
type initialExpert                                  index -> cert ->  seq -> o.
type  decideExpert                          index -> cert -> cert -> bseq -> o.
type   cutExpert form -> lincon -> lincon -> cert -> cert -> cert -> bseq -> o.
