module lkuTactics.



type orTac                                         choice -> seq -> seq -> o.
type existsTac                                          A -> seq -> seq -> o.
type oneTac                                                         seq -> o.
type initialTac                                            index -> seq -> o.
type decideTac                                    index -> bseq ->  seq -> o.
type releaseTac                                             seq -> aseq -> o.
type andTac                    lincon -> lincon ->  seq ->  seq ->  seq -> o.
type cutTac            form -> lincon -> lincon -> bseq -> aseq -> aseq -> o.

/* Checkers that translate information in the certificate to 
  positive proof rules. */
type bcheck 	cert -> bseq -> o.
type check 	cert ->  seq -> o.



end_sequent B (async Empty [B]) :- mset_empty Empty.


asyncTac Cert (async Delta [C|RHS]) :- 
   conj- A B C, andClerk C Cert CertL CertR,
   asyncTac CertL (async Delta [A|RHS]), 
   asyncTac CertR (async Delta [B|RHS]).
asyncTac Cert (async Delta [D|RHS]) :- 
   disj- A B D, orClerk D Cert Cert',
   asyncTac Cert' (async Delta [A,B|RHS]).  
asyncTac Cert (async Delta [U|RHS]) :- 
   all- A U, allClerk U Cert Cert',
   pi x\ asyncTac Cert' (async Delta [A x|RHS]).
asyncTac Cert (async Delta [F|RHS]) :- 
   false- F, falseClerk F Cert Cert',
   asyncTac Cert' (async Delta RHS).
asyncTac _ (async _  [T|_]) :- true- T.
asyncTac Cert (async Delta [C|RHS]) :- 
   store1 C Delta, store1Clerk C Idx Cert Cert', mset_add Idx Delta Delta', 
   ref Idx C => asyncTac Cert' (async Delta' RHS).
asyncTac Cert (async Delta [C|RHS]) :- 
   store2 C, store2Clerk C Idx Cert Cert',
   ref Idx C => czone Idx => asyncTac Cert' (async Delta RHS).
asyncTac Cert (async Delta nil) :- bcheck Cert (boundry Delta).

% Synchronous tactics (inference rules)
andTac DeltaL DeltaR (sync Delta C) (sync DeltaL A) (sync DeltaR B) :-
   conj+ A B C, mset_split Delta DeltaL DeltaR.
orTac left  (sync Delta D) (sync Delta A)     &
orTac right (sync Delta D) (sync Delta B)     :- disj+ A B D.
existsTac T (sync Delta U) (sync Delta (A T)) :- some+ A U.
oneTac      (sync Delta T)                    :- true+ T, mset_empty Delta.

% Some structural rules
initialTac Idx (sync Empty P) :- 
   mset_empty Empty, initial2 P, negate P NP,
   ref Idx NP, czone Idx.
initialTac Idx (sync Delta P) :- 
   initial1 P, negate P NP, 
   ref Idx NP, mset_singleton Idx Delta.
decideTac Idx (boundry Delta) (sync Delta  P) :- 
   czone Idx, ref Idx P, decide2 P.
decideTac Idx (boundry Delta) (sync Delta' P) :- 
   mset_select Idx Delta Delta', ref Idx P, decide1 P.
releaseTac (sync Delta N) (async Delta [N]) :- release1 N Delta.
releaseTac (sync Delta N) (async Empty [N]) :- mset_empty Empty, release2 N.
cutTac Cut DeltaL DeltaR (boundry Delta) 
   (async DeltaL [Cut]) (async DeltaR [NCut]) :- 
   negate Cut NCut, mset_split Delta DeltaL DeltaR.

% Checking a certificate against a sequent
check Cert Seq :- Seq = (sync _ N), isNeg N,
   releaseExpert Cert Cert' Seq, releaseTac Seq ASeq, asyncTac Cert' ASeq.
check Cert Seq :- Seq = (sync _ P), pos_lit P, 
   initialExpert Idx  Cert  Seq, initialTac Idx Seq.
check Cert Seq :- Seq = (sync _ One), true+ One, 
   trueExpert Cert Seq, oneTac Seq.  
check Cert Seq :- Seq = (sync _ Disj), disj+ _ _ Disj,
   orExpert Choice Cert Cert' Seq, orTac Choice Seq Seq', check Cert' Seq'.
check Cert Seq :- Seq = (sync _ Exists), some+ _ Exists,
   existsExpert  T Cert Cert' Seq, existsTac T  Seq Seq',
   check Cert' Seq'.
check Cert Seq :- Seq = (sync _ Conj), conj+ _ _ Conj,
   andExpert DeltaL DeltaR Cert CertL CertR Seq,
   andTac DeltaL DeltaR Seq SeqL SeqR,
   check CertL SeqL, check CertR SeqR.
bcheck Cert (boundry Delta) :- 
   decideExpert Idx Cert Cert' (boundry Delta),
   decideTac Idx (boundry Delta) Seq, 
   check Cert' Seq.
bcheck Cert (boundry Delta) :-
   cutExpert Cut DeltaL DeltaR Cert CertL CertR (boundry Delta), 	 
   cutTac Cut DeltaL DeltaR (boundry Delta) SeqL SeqR, 	 
   asyncTac CertL SeqL, 	 
   asyncTac CertR SeqR.

